# Search API Japanese Tokenizer

## 概要

日本語の文章を適切な単語単位で分割・インデックス化して検索性能を向上させる[Search API]用のモジュールです。通常、高度な日本語検索機能を実装する場合は[Search API]とApache SolrやElasticsearchなどの外部検索エンジンを組み合わせて実現しますが、本モジュールは[Search API]から機械学習ライブラリや外部の形態素解析器を利用してDrupalの検索機能を強化します。

## DrupalコアやSearch API標準の日本語検索の問題

日本語の文章は単語ごとにスペースで区切られていないため、単語ごとに分割することが難しい言語です。 DrupalコアのSearchモジュールやSearch APIで提供される日本語の検索機能では、N-gramと呼ばれる簡易的な処理を行って検索を行いますが、次の弱点があります。

- 1文字の単語をインデックス・検索することが事実上不可能
- 意図しない記事がヒットする可能性がある
- 完全に一致するフレーズしか検索できない

これらの問題は、自然言語処理で文章中の単語を意味がある単位に分割することで解決できます。

## サポートする自然言語処理アプリケーション

本モジュールでは次の自然言語処理アプリケーションをサポートしています。

- **[TinySegmenter]**: 機械学習を用いて日本語の文章を単語単位で分割するトークナイザーです。元はJavaScriptで書かれたライブラリですが、本モジュールではPHPに移植された[TinySegmenter PHP]を利用します。外部アプリケーションのインストールが不要なため手軽です
- **[MeCab]**: 形態素解析エンジンで品詞解析や原型変換などの機能を持っており、非常に高速に動作します。
- **[Sudachi]**: [MeCab]よりあとに開発された形態素解析器で、解析精度や機能が多いことが特徴です

|            | [TinySegmenter] | [MeCab]         | [Sudachi]  |
| ---------- | --------------- | --------------- | ---------- |
| 方法       | 機械学習        | 形態素解析      | 形態素解析 |
| 品詞解析   | なし            | あり            | あり       |
| 速度       | 速め            | 速い            | 遅い       |
| 精度       | 低い            | 普通            | 高い       |
| 正規化機能 | なし            | あり (動詞の原型化) | あり       |
| 外部インストール | 不要            | 必要            | 必要       |

[MeCab]や[Sudachi]はPHPアプリケーションではないため、別途サーバー上にインストールする必要があります。VPS等のアプリケーションを自由にインストールできる環境でないと利用できません。

各トークナイザーの要件やインストール方法、設定方法については、それぞれのサブモジュールのディレクトリにあるREADMEファイルを参照してください。

### モジュールのインストール

本モジュールを利用するには[Search API]モジュールが必要です。Search APIの設定方法については、[Search API モジュールのドキュメント](https://www.drupal.org/docs/contributed-modules/search-api)を参照してください。

対応するトークナイザーごとにサブモジュール化されています。必要なトークナイザーをインストールしてください。

## 設定

インストール後は `/admin/config/search/search-api` にアクセスし、Search APIの検索サーバーおよび検索インデックスを設定してください。インストールしたトークナイザーはプロセッサーの設定フォームで選択できます。フォームの説明に従って環境に合わせた設定を行ってください。設定の詳細は[Search API]モジュールのページを参照してください。

Search APIの標準トークナイザーと本モジュールのトークナイザーを同時に有効にすると、インデックス作成時に誤った分割が発生する可能性があります。そのため、Search APIの設定でTokenizerプロセッサーを無効化してください。

また、入力されるテキストデータの表記揺れを正規化することで、より自然言語処理の精度を向上させることができます。[Search API Japanese Normalizer](https://www.drupal.org/project/search_api_japanese_normalizer)モジュールの利用を検討してください。

## FAQ

### Q. 「持ち帰る」と言う文章が「帰る」で検索した時にヒットしません

Search APIの「サーバー」の設定で「Partial matching」の設定が「Match whole words only」になっていることが原因かもしれません。この設定は、インデックスされている単語に完全に一致するものしか検索にヒットしません。この設定を「Match on parts of a word」にすることで、部分一致でもヒットするようになります。ただし、この設定はパフォーマンス面の問題から非推奨です。

### Q. プロセッサーで設定した最小文字数の設定が反映されません

Search APIモジュールには、トークナイザー以外にも文字数の設定をするべき箇所が複数あります。次の設定も確認してください。

- Database Serverの文字数設定
- 検索画面Viewsのフィルター「Minimum keyword length」

### Q. 外国語が混ざったコンテンツが期待通りにインデックスされない

本モジュールのトークナイザーは日本語専用のため、外国語（英語・中国語など）が含まれるコンテンツを適切に処理できません。

## ライセンス

- 本モジュールはGPL v2ライセンスのもとで公開されています。詳細は[LICENSE.txt](./LICENSE.txt)を参照してください

## メンテナー

- [大野 裕太郎 (u7aro)]

## 支援団体

- [スタジオ・ウミ株式会社]

[Search API]: https://www.drupal.org/project/search_api
[TinySegmenter]: http://chasen.org/~taku/software/TinySegmenter/
[TinySegmenter PHP]: https://github.com/u7aro/tinysegmenter-php
[MeCab]: https://taku910.github.io/mecab/
[Sudachi]: https://github.com/WorksApplications/Sudachi
[大野 裕太郎 (u7aro)]: https://www.drupal.org/u/u7aro
[スタジオ・ウミ株式会社]: https://www.drupal.org/studio-umi
