# Search API Japanese Tokenizer

## Overview

This is a module for [Search API] that improves search performance by properly tokenizing and indexing Japanese text.
For advanced Japanese search functionality, the typical approach is to combine [Search API] with an external search engine like Apache Solr or Elasticsearch.
However, this module enhances Drupal's search capabilities by utilizing machine learning libraries or external morphological analyzers directly from [Search API].

## Issues with Drupal Core and Search API’s Built-in Japanese Search

Japanese text does not have spaces between words, making it difficult to tokenize properly.
Drupal Core’s Search module and Search API use a simple method called **N-gram** for Japanese text indexing, but this approach has the following limitations:

- It is practically impossible to index or search for single-character words.
- Irrelevant content may appear in search results.
- Only exact phrase matches work properly.

These issues can be resolved by using natural language processing (NLP) to segment sentences into meaningful words.

## Supported NLP Applications

This module supports the following NLP applications:

- **[TinySegmenter]**: A tokenizer that segments Japanese text using machine learning. Originally written in JavaScript, but this module uses a PHP port of **[TinySegmenter PHP]**. It does not require external applications, making it easy to use.
- **[MeCab]**: A morphological analysis engine that provides part-of-speech tagging and base form conversion. It operates at high speed.
- **[Sudachi]**: A more advanced morphological analysis engine developed after [MeCab], offering better accuracy and additional features.

|            | [TinySegmenter] | [MeCab]         | [Sudachi]  |
| ---------- | --------------- | --------------- | ---------- |
| Method     | Machine learning | Morphological analysis | Morphological analysis |
| POS Tagging | No              | Yes             | Yes        |
| Speed      | Fast            | Fast            | Slow       |
| Accuracy   | Low             | Medium          | High       |
| Normalization | No            | Yes (Base form conversion) | Yes |
| External Application | Not required | Required | Required |

Since [MeCab] and [Sudachi] are not PHP applications, they must be installed separately on the server.
They can only be used in environments where you have control over server applications (e.g., VPS).
For requirements, installation, and configuration instructions for each tokenizer, refer to the README file in each submodule directory.

### Installation

To use this module, you need to install the [Search API] module.
Refer to the [Search API module documentation](https://www.drupal.org/docs/contributed-modules/search-api) for installation instructions.

Each tokenizer is provided as a separate submodule.
Install the submodule corresponding to the tokenizer you want to use.

## Configuration

After installation, go to `/admin/config/search/search-api` and configure your Search API **server** and **index**.
The installed tokenizer can be selected from the processor settings form.
Follow the form instructions to configure the settings according to your environment.
For more details, refer to the [Search API] module page.

⚠ **Warning:**
If you enable both the default Search API tokenizer and this module’s tokenizer simultaneously, incorrect tokenization may occur when creating the search index.
To prevent this, disable the **Tokenizer processor** in the Search API settings.

Additionally, **text normalization** can improve NLP accuracy by reducing inconsistencies in input text.
Consider using the [Search API Japanese Normalizer](https://www.drupal.org/project/search_api_japanese_normalizer) module.

## FAQ

### Q. Why doesn't the word "帰る" match when searching for "持ち帰る"?

This might be due to the **Partial matching** setting in the Search API server configuration.
If it is set to **"Match whole words only"**, only exact matches will be found.
Changing this setting to **"Match on parts of a word"** allows partial matches.

However, this setting is **not recommended** due to performance concerns.

### Q. The minimum character length setting in the processor is not being applied. Why?

There are multiple places where character length settings need to be configured in the Search API module. Check the following settings:

- Minimum character length setting in the **Database Server**
- **Views filter** for search pages: "Minimum keyword length"

### Q. The index is not handling multilingual content correctly.

This module’s tokenizers are designed for **Japanese only**.
They may not work properly with content that contains foreign languages (e.g., English, Chinese, etc.).

## License

- This module is released under the **GPL v2 license**.
  See [LICENSE.txt](./LICENSE.txt) for details.

## Maintainer

- [Yutaro Ohno (u7aro)]

## Supporting Organization

- [Studio Umi, Inc.]

[Search API]: https://www.drupal.org/project/search_api
[TinySegmenter]: http://chasen.org/~taku/software/TinySegmenter/
[TinySegmenter PHP]: https://github.com/u7aro/tinysegmenter-php
[MeCab]: https://taku910.github.io/mecab/
[Sudachi]: https://github.com/WorksApplications/Sudachi
[Yutaro Ohno (u7aro)]: https://www.drupal.org/u/u7aro
[Studio Umi, Inc.]: https://www.drupal.org/studio-umi
