<?php

namespace Drupal\search_api_mecab;

use Drupal\search_api_japanese_tokenizer\MaManagerInterface;

/**
 * Interface for MeCab command wrapped class.
 */
interface MecabManagerInterface extends MaManagerInterface {

  /**
   * Returns MecabWord objects.
   *
   * @return \Drupal\search_api_mecab\MecabWord[]
   *   MecabWord objects.
   */
  public function getWords(): array;

  /**
   * Set MeCab executable directory path.
   *
   * @param string $path
   *   MeCab executable directory path.
   *
   * @return static
   *   The called object.
   */
  public function setExecutableDir(string $path): static;

  /**
   * Set system dictionary directory path.
   *
   * @param string $path
   *   System dictionary directory path.
   *
   * @return static
   *   The called object.
   */
  public function setSystemDictionaryDir(string $path): static;

  /**
   * Return system installed MeCab version.
   *
   * @return string
   *   Version number of system installed MeCab.
   */
  public function getVersion(): string;

}
