<?php

namespace Drupal\search_api_sudachi;

use Drupal\search_api_japanese_tokenizer\MaManagerInterface;

/**
 * Interface for morphological analysis manager.
 */
interface SudachiManagerInterface extends MaManagerInterface {

  /**
   * Sets the path to the Sudachi JAR file.
   *
   * @param string $path
   *   The path to the Sudachi JAR file.
   *
   * @return static
   *   Returns the current instance for method chaining.
   */
  public function setJarPath(string $path): static;

  /**
   * Sets the path to the Sudachi settings file.
   *
   * @param string $path
   *   The path to the Sudachi settings file.
   *
   * @return static
   *   Returns the current instance for method chaining.
   */
  public function setSettingsFilePath(string $path): static;

  /**
   * Sets the path to the Sudachi system dictionary.
   *
   * @param string $path
   *   The path to the Sudachi system dictionary.
   *
   * @return static
   *   Returns the current instance for method chaining.
   */
  public function setSystemDictionaryPath(string $path): static;

  /**
   * Sets the split mode.
   *
   * @param string $mode
   *   The split mode.
   *
   * @return static
   *   Returns the current instance for method chaining.
   */
  public function setSplitMode(string $mode): static;

  /**
   * Get Sudachi words.
   *
   * @return \Drupal\search_api_sudachi\SudachiWordInterface[]
   *   Sudachi words.
   */
  public function getWords(): array;

}
