<?php

namespace Drupal\search_api_sudachi\Utility;

use Drupal\search_api_sudachi\Model\SudachiWord;
use Drupal\search_api_sudachi\SudachiWordInterface;

/**
 * Provides parsing Sudachi output line.
 */
class SudachiParser {

  const FEATURE_PART_OF_SPEECH = 0;

  /**
   * Parse analysis features.
   *
   * @param string $features_csv
   *   Features CSV string.
   *
   * @return array
   *   Array of features.
   */
  protected static function parseAnalysisFeaturesCsv(string $features_csv): array {
    return explode(',', $features_csv);
  }

  /**
   * Parse line text of Sudachi output.
   *
   * @param string $line
   *   Line text of Sudachi output.
   *
   * @return \Drupal\search_api_sudachi\SudachiWordInterface
   *   SudachiWord object.
   */
  protected static function parseAnalysisLine(string $line): SudachiWordInterface {
    // Sudachi output line format:
    // `{surface}\t{comma_separated_features}\t{base_form}`.
    // Ex: `生まれ\t動詞,一般,*,*,下一段-ラ行,連用形-一般\t生まれる`.
    [$surface, $features_csv, $normalized_word] = explode("\t", $line);
    $features = self::parseAnalysisFeaturesCsv($features_csv);
    $sudachi_word = new SudachiWord();
    $sudachi_word->setSurface($surface);
    $sudachi_word->setPartOfSpeech($features[self::FEATURE_PART_OF_SPEECH]);
    $sudachi_word->setNormalizedWord($normalized_word);

    return $sudachi_word;
  }

  /**
   * Parse text of Sudachi analysis output.
   *
   * @param string $text
   *   Text of Sudachi output.
   *
   * @return \Drupal\search_api_sudachi\SudaChiWordInterface[]
   *   Array of SudachiWord objects.
   */
  public static function parseAnalysisOutput(string $text): array {
    // dpm($text);
    $lines = explode("\n", $text);
    $lines = array_filter($lines, function ($line) {
      return $line != 'EOS';
    });
    return array_map([self::class, 'parseAnalysisLine'], $lines);
  }

}
