<?php

namespace Drupal\search_api_tinysegmenter\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api_japanese_tokenizer\Form\JapaneseTokenizerTestFormBase;
use U7aro\TinySegmenter\TinySegmenter;

/**
 * Defines a form for testing the TinySegmenter integration.
 */
class TinySegmenterTestForm extends JapaneseTokenizerTestFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tinysegmenter_test_form';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild(TRUE);

    $text = $form_state->getValue('input_text');
    $text = preg_replace('/\s+/', ' ', $text);
    try {
      $start_time = microtime(TRUE);
      $words = (new TinySegmenter())->segment($text);
      $end_time = microtime(TRUE);

      $form_state->set('tokenized_text', implode(self::WORD_DELIMITER, $words));
      $execution_time = ($end_time - $start_time);
      $this->messenger()->addStatus($this->t('Tokenization completed in @time seconds.', ['@time' => number_format($execution_time, 2)]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An error occurred while analyzing the text: @message', ['@message' => $e->getMessage()]));
    }
  }

}
