<?php

namespace Drupal\search_api_tinysegmenter\Plugin\search_api\processor;

use Drupal\search_api_japanese_tokenizer\Plugin\search_api\processor\MlTokenizerBase;
use U7aro\TinySegmenter\TinySegmenter;

/**
 * Splits text into individual words for searching.
 *
 * @SearchApiProcessor(
 *   id = "tinysegmenter_tokenizer",
 *   label = @Translation("TinySegmenter tokenizer"),
 *   description = @Translation("Extend the functionality of the Tokenizer to perform tokenization of Japanese text using TinySegmenter. Since the traditional CJK processing cannot be used concurrently, issues may occur in sentences containing Chinese or Korean text."),
 *   stages = {
 *     "pre_index_save" = 0,
 *     "preprocess_index" = -6,
 *     "preprocess_query" = -6
 *   }
 * )
 */
class TinySegmenterTokenizer extends MlTokenizerBase {

  /**
   * Segments Japanese text into individual tokens using TinySegmenter.
   *
   * @param array $matches
   *   The matched array from `preg_replace_callback`. The first element
   *   ($matches[0]) contains the matched Japanese text to be segmented.
   *
   * @return string
   *   The tokenized string with individual tokens separated by spaces.
   */
  protected function segmentJapaneseText($matches): string {
    $words = TinySegmenter::segment($matches[0]);
    return ' ' . implode(' ', $words) . ' ';
  }

  /**
   * {@inheritdoc}
   */
  protected function tokenizeText(string $value): array {
    // TinySegmenter Ver. 0.2 has an issue where numbers with two or more digits
    // are split into individual digits. Therefore, the process is limited to
    // Japanese text only.
    $text = preg_replace_callback(
      '/[' . self::PREG_JAPANESE_CLASS . ']+/u',
      [$this, 'segmentJapaneseText'],
      $value
    );

    $tokens = preg_split('/\s+/', trim($text));
    $tokens = array_filter($tokens, [$this, 'isIndexableWord']);

    return $tokens;
  }

}
