<?php

namespace Drupal\search_api_japanese_tokenizer\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a base form for testing Japanese text tokenization.
 *
 * This abstract form allows developers to input Japanese text and view
 * the tokenized output using a tokenizer library.
 */
abstract class JapaneseTokenizerTestFormBase extends FormBase {

  /**
   * The character used to separate tokens in the tokenized output.
   */
  const WORD_DELIMITER = '|';

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attached']['library'][] = 'search_api_japanese_tokenizer/test-form';

    $form['test_form'] = [
      '#prefix' => '<div class="search-api-japanese-tokenizer-test-form">',
      '#suffix' => '</div>',
    ];

    $form['test_form']['input_text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Input text for tokenization'),
      '#description' => $this->t('Enter Japanese text to tokenize into individual words or phrases.'),
      '#required' => TRUE,
      '#prefix' => '<div class="search-api-japanese-tokenizer-test-form__input-text">',
      '#suffix' => '</div>',
      '#rows' => 10,
    ];

    $tokenized_text = $form_state->get('tokenized_text');
    $form['test_form']['tokenized_text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Tokenized output'),
      '#description' => $this->t('This field displays the tokenized version of the input text. Tokens are separated using the "@delimiter" character.', ['@delimiter' => static::WORD_DELIMITER]),
      '#default_value' => $tokenized_text,
      '#disabled' => TRUE,
      '#prefix' => '<div class="search-api-japanese-tokenizer-test-form__tokenized-text">',
      '#suffix' => '</div>',
      '#rows' => 10,
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Perform tokenization'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

}
