<?php

namespace Drupal\search_api_japanese_tokenizer;

/**
 * Interface for morphological analysis manager.
 */
interface MaManagerInterface {

  /**
   * Analyze text by morphological analysis.
   *
   * @param string $text
   *   Text to analyze.
   *
   * @return static
   *   Return self object.
   */
  public function analyze(string $text): static;

  /**
   * Return raw output of morphological analysis.
   *
   * @return string
   *   Raw output of morphological analysis.
   */
  public function getRawOutput(): string;

  /**
   * Return MaWord interface objects.
   *
   * @return \Drupal\search_api_japanese_tokenizer\MaWordInterface[]
   *   MaWord interface objects.
   */
  public function getWords(): array;

  /**
   * Check if morphological analysis command is executable.
   *
   * @return bool
   *   Return true if morphological analysis command is executable.
   */
  public function isExecutable(): bool;

}
